%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   Transport a light-field given in direction major format a certain
%   distance d in free space.
%
%   input:  lfDM    -   light-field in direction major format 
%                       lfDM is a matrix of subimages that each represent
%                       a fixed angle (theta, phi) 
%           thetas  -   row vector with values for theta for each column of
%                       the LF subimages (angle in x)
%           phis    -   column vector with a value for phi for each row of
%                       the LF subimages (angle in y)
%           d       -   the distance to travel
%
%   optional:
%           xx      -   x coordinates for each pixel within the subimages
%                       in some unit. depends on the sensor size and the
%                       sensor resolution. default: -1: :1
%           yy      -   y coordinates for each pixel within the subimages
%                       in some unit. depends on the sensor size and the
%                       sensor resolution. default: -1: :1
%       
%   output: lfDMNew -   light-field propagated a distance d             
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   September, 2009
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lfDMNew = lightFieldTransport(lfDM, thetas, phis, d, xGrid, yGrid)

    % get the number of different subimages
    numSubimages = [ numel(phis) numel(thetas) ];
    % get size of subimages
    subimageSize = [ size(lfDM,1)/numSubimages(1) size(lfDM,2)/numSubimages(2) ];
    
    % initialize output
    lfDMNew = zeros(size(lfDM));
    
    if nargin < 5
        % get sampling grid
        [xGrid yGrid] = meshgrid(   double(0:subimageSize(2)-1)./double(subimageSize(2)) - double(subimageSize(2))/2, ...
                                    double(0:subimageSize(1)-1)./double(subimageSize(1)) - double(subimageSize(1))/2 );
    end
    
    % over all subimages
    for y=1:numSubimages(1)
        for x=1:numSubimages(2)
       
            % for each color channel
            for c=1:size(lfDM,3)
                
                % extract current subimage
                currentLFSlice = lfDM( (y-1)*subimageSize(1)+1:y*subimageSize(1),...
                                       (x-1)*subimageSize(2)+1:x*subimageSize(2), c);
                                                 
                % simulate propagation through interpolation
                propagatedLFSlice = interp2(xGrid, yGrid,  currentLFSlice, ...
                                            xGrid - d*tan(thetas(x)), yGrid - d*tan(thetas(y)), ...
                                            'cubic', 0.0);
                                        
                % sort in new image
                lfDMNew(    (y-1)*subimageSize(1)+1:y*subimageSize(1),...
                            (x-1)*subimageSize(2)+1:x*subimageSize(2), c) = propagatedLFSlice;
            end  
            
%             imshow(lfDMNew);
%             drawnow;
            
        end
    end

end