%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   This script loads a sensor image from a specified light field camera
%   type, adds the specified noise, and reconstructs the light field which
%   includes the propagated sensor noise. This precedure can take quite
%   long for the non-preview mode!
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   March 2010
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function reconstructLightFieldFromSensorImage(  datapath, filename, acquisitionMethod,...
                                                modulatorSize, windowSize, bUpsampleMeasurementChannels,...
                                                bPreviewMode, upsamplingMethod,...
                                                additiveNoise, photonNoise)

% approximate image intensities for photon noise with mean light
% transmission of modulator
bApproximatePhotonNoiseWithMeanMaskTransmission = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the modulator

% lenslets
if acquisitionMethod == 0
    
    modulator = zeros(modulatorSize);
    % set the central pixel to non 0
    modulator(1,1) = (pi/4) * modulatorSize^2; 

    method = 'LensletArray';
    
% pinholes    
elseif acquisitionMethod == 1
    
    modulator = zeros(modulatorSize);
    % set the central pixel to 1
    modulator(1,1) = 1; 

    method = 'PinholeArray';
% SoS    
elseif acquisitionMethod == 2
    modulator = sos(modulatorSize);
    method = 'SoS';
        
% MURA    
elseif acquisitionMethod == 3
    modulator = ifftshift(mura(modulatorSize));
    method = 'MURA';

end

modulator = fftshift(modulator);
modulator = modulator ./ (modulatorSize*modulatorSize);

% get Fourier transform
modulatorOTF = (psf2otf(modulator));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load sensor image from file
load([datapath '/' filename '_SensorImage_' method], 'sensorImage');

% simulate noise
if bApproximatePhotonNoiseWithMeanMaskTransmission == 0
    sensorImage = sensorImage + photonNoise.*sensorImage.*randn(size(sensorImage)) + additiveNoise.*randn(size(sensorImage));
else
    tau = mean(mean(modulator.*modulatorSize^2));
    sensorImage = sensorImage + photonNoise*tau.*randn(size(sensorImage)) + additiveNoise.*randn(size(sensorImage));
end
sensorImage(sensorImage>1) = 1;
sensorImage(sensorImage<0) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% upsample each measurement channel to full image resolution - save to
% files

%dirname = [datapath '/' filename '_SensorImage_' method '_Reconstructed_Noise_' num2str(additiveNoise) '_' num2str(photonNoise) '_' upsamplingMethod];
if bPreviewMode == 0
    dirname = [datapath '/' filename '_SensorImage_' method '_Reconstructed_Noise_' num2str(additiveNoise) '_' num2str(photonNoise) '_' upsamplingMethod];
else
    dirname = [datapath '/' filename '_SensorImage_' method '_Reconstructed_Noise_Preview_' num2str(additiveNoise) '_' num2str(photonNoise) '_' upsamplingMethod];
end
[a b c] = mkdir(dirname);

if ~bPreviewMode
    % save noisy sensor image
    imwrite(sensorImage, [dirname '/' filename '_SensorImage_' method '_Noise_' num2str(additiveNoise) '_' num2str(photonNoise) '.png']);    
end


% get the measurement channels, but save them into separate files!
if bUpsampleMeasurementChannels ~= 0

    % coordinates for c
    [xGridC yGridC] = meshgrid(1:size(sensorImage,2), 1:size(sensorImage,1));

    % coordinates for upper left pixel within each super-pixel
    xx = 1:modulatorSize:size(sensorImage,2);
    yy = 1:modulatorSize:size(sensorImage,1);
    [xGridS yGridS] = meshgrid(xx, yy);

    disp(['  Writing upsampled measurement channels']);

    % over all indices within a super-pixel
    k=1;
    for spy=1:modulatorSize
        for spx=1:modulatorSize
            
            % upsample 
            currentChannel = interp2(   xGridS+spx-1, yGridS+spy-1, sensorImage(yy+spy-1,xx+spx-1), ...
                                        xGridC, yGridC, upsamplingMethod, 0.0);
                                    
                            
            % only use the center of each super-pixel for the preview mode
            if bPreviewMode ~= 0
                currentChannel = currentChannel(yy+4, xx+4);
            end
                                                
            % clamp negative values for cubic upsampling
            if (strcmp(upsamplingMethod, 'cubic') == 1) || (strcmp(upsamplingMethod, 'bicubic') == 1)
                currentChannel(currentChannel<0) = 0;
            end
                                            
            % save channel
            save([dirname '/view_' num2str(spy) '_' num2str(spx) '.mat'], 'currentChannel');
                        
            disp(['    ' num2str(k) ' of ' num2str(modulatorSize^2)]);
            k=k+1;
        end
    end

    clear xx yy xGridC y GridC xGridS yGridS currentChannel;

    disp(['  done'])
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now separate the sensor image into windows and for each pixel of each
% window perform the decorrelation

% only one window for preview mode
if bPreviewMode ~= 0
    numWindows = [1 1]; 
    windowSize = size(sensorImage) ./ modulatorSize;    
else
    numWindows = size(sensorImage) ./ windowSize;
end


disp(['  Processing windows for decorrelation']);

k=1;
for wy=1:numWindows(1)
    for wx=1:numWindows(2)    
        
        disp(['    ' num2str(k) ' of ' num2str(numWindows(1)*numWindows(2))]);
   
        % measurement channels
        c = zeros([windowSize(1) windowSize(2) modulatorSize modulatorSize]);
        
        % reconstructions
        l = zeros([windowSize(1) windowSize(2) modulatorSize modulatorSize]);
        
        disp('       loading stack');
        
        % load all windows into stack
        for sy=1:modulatorSize
            for sx=1:modulatorSize
  
                % load current view
                load([dirname '/view_' num2str(sy) '_' num2str(sx) '.mat'], 'currentChannel');
                
                % crop current window from that view and save it in c
                c(:,:,sy,sx) = currentChannel( (wy-1)*windowSize(1)+1:wy*windowSize(1), (wx-1)*windowSize(2)+1:wx*windowSize(2) );
            
            end
        end
                
        disp('       performing decorrelation');
        
        % perform decorrelation for each pixel
        for py=1:windowSize(1)
            for px=1:windowSize(2)
                
                % reshape current pixel
                tile = reshape(c(py,px,:,:), [modulatorSize modulatorSize]);
                                
                % perform decorrelation 
                tile = ifft2( fft2(tile) ./ modulatorOTF );           
                
                % write back
                l(py,px,:,:) = reshape(tile, [1 1 modulatorSize modulatorSize]);
                
            end
        end
        
        disp('       writing result');
        
        
        if bPreviewMode ~= 0
        
            % just show one of the reconstructed views
            imshow(l(:,:,5,5))
            [s name] = system('hostname');
            title(['method: ' num2str(acquisitionMethod) ' at ' name]);
            imwrite(l(:,:,5,5), [dirname '/reconstruction_preview.png']);
        
        % write windows back to files   
        else
            
            for sy=1:modulatorSize
                for sx=1:modulatorSize

                    if k==1
                        I = zeros(size(sensorImage));
                    else
                        %I = double(imread([dirname '/reconstruction_' num2str(sy) '_' num2str(sx) '.png'])) ./ 255;
                        load([dirname '/reconstruction_' num2str(sy) '_' num2str(sx) '.mat'], 'I');
                    end

                    % set window to reconstruction
                    I( (wy-1)*windowSize(1)+1:wy*windowSize(1), (wx-1)*windowSize(2)+1:wx*windowSize(2) ) = ...
                        l(:,:,sy,sx);                                   

                    % save file                
                    save([dirname '/reconstruction_' num2str(sy) '_' num2str(sx) '.mat'], 'I');
                end
            end
        end
        
        k = k+1;
    end
end

if bPreviewMode == 0
    % save the desired view as image
    load([dirname '/reconstruction_' num2str(5) '_' num2str(5) '.mat'], 'I');
    imwrite(I, [dirname '/reconstruction_' num2str(5) '_' num2str(5) '.png']);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
              
                