%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   This demo simulates noisy sensor image for different light field 
%   cameras by loading an existing light field and simulating its capture
%   with a lens- or mask-based light field camera.  
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   March 2010
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

% path to light field data
datapath = '../data';

% filename
filename = 'Golgi11x11';

% check if file exists
if (exist([datapath '/' filename '.png'], 'file') == 0) || (exist([datapath '/' filename 'DM.png'], 'file') == 0)
	error(['File ' filename ' does not exist. Make sure to follow the instructions in ../README!']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Type of modulator:
%   0 - lenslet array (this is the reference, so nothing is changing)
%   1 - pinhole array (attenuation mask)
%   2 - sum-of-sinusoids array (attenuation mask)
%   3 - MURA array (attenuation mask)
acquisitionMethods  = [0 1 2 3];

% size of one period of the modulator in pixels of light-field
modulatorSize       = 11;

% The preview mode is MUCH faster, because it only processes one pixel per 
% super-pixel. Although being faster, this does not apply our theory and
% only gives you an idea how the final result will look like at a lower 
% resolution.
% All results in the paper are generated in non-preview mode!
bPreviewMode = 1;

% For non-preview mode, the data is first upsampled, and then decorrelated
% on pixel-by-pixel basis. Due to memory restrictions, this is being
% performed in a windowed more. Define the size of one window here.
windowSize          = [187 275];

% upsampling method
upsamplingMethod = 'bicubic'; % this clearly gives the least aliasing artifacts!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% noise parameters

% one fixed additive noise term
additiveNoise = 0.4^2/255;

% different photon noise levels
noise = [ (1/53)/255  10 ^ ( log10(additiveNoise) + 1 )   10 ^ ( log10(additiveNoise) + 2 ) ] ; 

% run for all modulation methods
for m=1:size(acquisitionMethods,2)
    
    % current acquisition method
    acquisitionMethod = acquisitionMethods(m);
               
    % simulate the captured sensor image with this light field camera
    disp(['Simulating sensor image for light field camera of type ' num2str(acquisitionMethod)  '. This may take a minute or two...']);
    simulateLightFieldCamera(   datapath, filename, ...
                                acquisitionMethod, modulatorSize, 1);

    % run for all noise settings
    for k=1:size(noise,2)
            
        % current photon noise level
        photonNoise = noise(k);
        
        % reconstruct the light field
        reconstructLightFieldFromSensorImage(   datapath, filename, acquisitionMethod,...
                                                modulatorSize, windowSize, 1,...
                                                bPreviewMode, upsamplingMethod,...
                                                additiveNoise, photonNoise);
    end
end

