%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   Load a light field from files and simulate its capture with a
%   mask-based or lenslet camera.
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   September, 2009
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function simulateLightFieldCamera(  datapath, ...
                                    filename, ...
                                    acquisitionMethod, ...
                                    modulatorSize,...
                                    bSave)                                                

    % Three different method to create the sensor image (#3 is the one used in the paper):
    %   1 - sensor image has same size as dataset - assumes that LF does not 
    %       vary spatially over one lenslet
    %   2 - sensor image has resolution of #lenslets in dataset - more
    %       correctly
    %   3 - sensor image has same size as dataset - same as #1, except that the
    %       light-field is not spatially constant over each super-pixel, but
    %       interpolated
    sensorImageOption = 3;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % specific options for sensorImageOption 3!
    %
    % the band-limitation of the individual views is VERY important, otherwise
    % the reconstruction won't be recognizable at all due to the aliasing
    % artifacts!

    % upsampling method 
    %upsampling = 'nearest';
    %upsampling = 'bilinear';
    upsampling = 'bicubic';

    % band-limit filter is a Gauss, here the parameters
    gaussSize       = [100 100];    % 100x100 is large enough for a variance of 11
    gaussVariance   = 9;            % 11 is pretty blurry, but gives aliasing-free reconstructions
                                    % if done with bicubic upsampling

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % lenslets
    if acquisitionMethod == 0

        modulator = zeros(modulatorSize);
        % set the central pixel to non 0
        modulator(1,1) = (pi/4) * modulatorSize^2; 

        method = 'LensletArray';

    % pinholes    
    elseif acquisitionMethod == 1

        modulator = zeros(modulatorSize);
        % set the central pixel to 1
        modulator(1,1) = 1; 

        method = 'PinholeArray';
    % SoS    
    elseif acquisitionMethod == 2
        modulator = sos(modulatorSize);
        method = 'SoS';    

    % MURA    
    elseif acquisitionMethod == 3

        modulator = ifftshift(mura(modulatorSize));    
        method = 'MURA';
    end
    
	% check if the file exists already (takes too long to do it every time!)
    if exist([datapath '/' filename '_SensorImage_' method '.mat'], 'file') ~= 0
        disp(['  Sensor image exists already, using that one.']);
        return;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   load light-field and simulate modulator  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    LF = double(imread([datapath '/' filename '.png'])) ./ 255;

    % number of lenslets
    numLenslets = [size(LF,1)/modulatorSize size(LF,2)/modulatorSize];

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % sensor image option 1
    if sensorImageOption == 1

        sensorImage = zeros(size(LF));

        % make modulator suitable for convolution
        modulator = fftshift(modulator);
        modulator = modulator ./ (11*11);

        for ly=1:numLenslets(1)
            for lx=1:numLenslets(2)

                % extract current lenslet
                lenslet = LF(   (ly-1)*modulatorSize+1:ly*modulatorSize, ...
                                (lx-1)*modulatorSize+1:lx*modulatorSize, :);

                % simulate mask
                lensletNew = imfilter(lenslet, modulator, 'conv', 'circ');
                sensorImage(    (ly-1)*modulatorSize+1:ly*modulatorSize, ...
                                (lx-1)*modulatorSize+1:lx*modulatorSize, :) = lensletNew;

                sensorImage(    (ly-1)*modulatorSize+1:ly*modulatorSize, ...
                                (lx-1)*modulatorSize+1:lx*modulatorSize, :) = lensletNew;
            end
        end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

    elseif sensorImageOption == 2


        % simulated sensor image
        sensorImage = zeros(numLenslets);
        modulator = circshift(modulator, [-1 -1]);        

        for ly=1:numLenslets(1)
            for lx=1:numLenslets(2)

                % extract current lenslet
                lenslet = LF(   (ly-1)*modulatorSize+1:ly*modulatorSize, ...
                                (lx-1)*modulatorSize+1:lx*modulatorSize, :);

                % shift modulator
                currentModulator = circshift(modulator, [-(ly-1) -(lx-1)]);

                k = fspecial('gaussian', [11 11], 2); k = k ./ max(k(:));
                lenslet = lenslet .* k;

                % write back to light-field
                sensorImage(ly,lx) = mean(mean( lenslet .* currentModulator ));           
            end
        end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    elseif sensorImageOption == 3       

        % load direction-major view of light field
        LF = double(imread([datapath '/' filename 'DM.png'])) ./ 255;

        % initialize sensor image
        sensorImage = zeros(size(LF));

        % size of the upsampled and resulting sensor image for method 3
        method3SensorSize = [size(LF,1) size(LF,2)]; 

        % simulate attenuation mask
        modulator = fftshift(modulator) ./ (11*11);
        attenuationMask = repmat( modulator, method3SensorSize ./ modulatorSize );

        % for each view
        disp('  processing views')
        k=1;
        for vy=1:modulatorSize
            for vx=1:modulatorSize

                % extract current view
                currentView = LF(   (vy-1)*numLenslets(1)+1:vy*numLenslets(1), ...
                                    (vx-1)*numLenslets(2)+1:vx*numLenslets(2), :);

                % upsample view to full resolution            
                currentView = imresize(currentView, method3SensorSize, upsampling);

                currentView(currentView<0) = 0;
                currentView(currentView>1) = 1;

                % band-limit the view
                currentView     = imfilter(currentView, fspecial('gaussian', gaussSize, gaussVariance), 'conv', 'circ');                       

                % determine shift of mask for current direction
                maskShift   = [ -floor(modulatorSize/2)+vy-1 -floor(modulatorSize/2)+vx-1 ];

                % multiply with shifted mask and add to sensor image
                sensorImage = sensorImage + currentView .* circshift(attenuationMask, maskShift);                      

                disp(['    ' num2str(k) ' of ' num2str(modulatorSize^2)]);
                k=k+1;
            end

%             imagesc(sensorImage);
%             colormap gray;
%             drawnow;

        end

    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %   save simulated sensor image
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    sensorImage(sensorImage<0) = 0;
    
    % save sensor image
    if bSave ~= 0
        save([datapath '/' filename '_SensorImage_' method], 'sensorImage');
        imwrite(sensorImage, [datapath '/' filename '_SensorImage_' method '.png']);
    end
    
end