%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   Convert a light-field given in micro-lens or pinhole view to
%   direction-major format by resorting and interpolating the pixels.
%
%   input:  lfSM    - light-field in microlens view
%           N       - number of lenslets in [y x]
%           bDraw   - (optional) draw the image while its processing
%                       default is no draw
%           intMeth - (optional) interpolation method, one of 
%                       'nearest', 'linear', or 'cubic'
%                       default is nearest
%
%   output: lfDM    - light-field in direction major format
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   September, 2009
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function lfDM = lightFieldSpatialMajor2DirectionMajor(lfSM, N, bDraw, intMeth)

    % draw or not
    if nargin < 3
        bDraw = 0;
    end

    % interpolation method - 'nearest', 'linear', or 'cubic'
    if nargin < 4
        intMeth = 'nearest';
    end

    % number of different views
    numViews = [ floor(size(lfSM,1)/N(1)) floor(size(lfSM,2)/N(2)) ];
       
    % check if you need interpolation at all
    bNeedInterpolation = 1;
    if (size(lfSM,1)/N(1) == numViews(1)) & (size(lfSM,2)/N(2) == numViews(2))
        bNeedInterpolation = 0;
    end
    
    % interpolate
    if bNeedInterpolation
    
        % sampling grid
        ySamples = (1:size(lfSM,1)/N(1):size(lfSM,1))';
        xSamples = 1:size(lfSM,2)/N(2):size(lfSM,2);                      

        % initialize image
        lfDM = zeros([ numViews(1)*numel(ySamples) numViews(2)*numel(xSamples) size(lfSM,3)]);

        % for each view
        for y=0:numViews(1)-1
            for x=0:numViews(2)-1

                % for each color channel
                for c=1:size(lfSM, 3)

                    lfDM(   y*numel(ySamples)+1:(y+1)*numel(ySamples), ...
                            x*numel(xSamples)+1:(x+1)*numel(xSamples), c ) = ... 
                        interp2(lfSM(:,:,c), xSamples+x, ySamples+y, intMeth);      

                end

                if bDraw            
                    imshow(lfDM);
                    drawnow;             
                end
            end
		disp(['s2d - ' num2str(y) ' of ' num2str(numViews(1)-1)]);
        end
        
    % don't need interpolation
    else
        
        % initialize image
        lfDM = zeros([ N(1) N(2) size(lfSM,3)]);
        
        % sampling grid
        ySamples = 1:size(lfSM,1)/N(1):size(lfSM,1);
        xSamples = 1:size(lfSM,2)/N(2):size(lfSM,2); 

        % for each view
        for y=0:numViews(1)-1
            for x=0:numViews(2)-1

                % for each color channel
                for c=1:size(lfSM, 3)

                    lfDM(   y*numel(ySamples)+1:(y+1)*numel(ySamples), ...
                            x*numel(xSamples)+1:(x+1)*numel(xSamples), c ) = ... 
                        lfSM(ySamples+y,xSamples+x,c);     

                end

                if bDraw            
                    imshow(lfDM);
                    drawnow;             
                end
            end
        end
        
    end
end
